@@ -32,7 +32,7 @@ dfm <- function(x, ...) {
 #' @param verbose display messages if \code{TRUE}
 #' @param toLower convert texts to lowercase
 #' @param removeNumbers remove numbers, see \link{tokenize}
-#' @param removePunct remove numbers, see \link{tokenize}
+#' @param removePunct remove punctuation, see \link{tokenize}
 #' @param removeTwitter if \code{FALSE}, preserve \code{#} and \code{@@} 
 #'   characters, see \link{tokenize}
 #' @param removeSeparators remove separators (whitespace), see \link{tokenize}
@@ -168,6 +168,9 @@ dfm.character <- function(x,
     startTime <- proc.time()
     valuetype <- match.arg(valuetype)
     
+#     if (length(addedArgs <- list(...)) && !(addedArgs %in% names(formals(getS3method("tokenize", "character")))))
+#         warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
+
     if (verbose && grepl("^dfm\\.character", sys.calls()[[2]]))
         cat("Creating a dfm from a character vector ...")
 
@@ -237,10 +240,6 @@ dfm.tokenizedTexts <- function(x,
         cat("Creating a dfm from a tokenizedTexts object ...")
     }
 
-    # add a "NULL" token to every tokenized text, in case some are empty
-    x <- lapply(x, function(t) t <- c(t, "**_NULL_**"))
-    class(x) <- c("tokenizedTexts", "list")
-    
     # get document names
     if (is.null(names(x))) {
         docNames <- paste("text", 1:length(x), sep="")
@@ -251,8 +250,11 @@ dfm.tokenizedTexts <- function(x,
                      format(length(x), big.mark=","), " document",
                      ifelse(length(x) > 1, "s", ""), sep="")
     nTokens <- lengths(x)
-    docIndex <- rep(seq_along(nTokens), nTokens)
-    
+    # find out which documents have zero feature counts
+    emptyDocs <- which(nTokens == 0)
+    # add docIndex positions for any zero-token docs; no effect if emptyDocs is empty
+    docIndex <- c(rep(seq_along(nTokens), nTokens), emptyDocs)
+
     # index features
     if (verbose) cat("\n   ... indexing features: ")
     allFeatures <- unlist(x)
@@ -261,6 +263,11 @@ dfm.tokenizedTexts <- function(x,
     if (verbose) cat(format(totalfeatures - 1, big.mark=","), " feature type",
                      ifelse(totalfeatures - 1  > 1, "s", ""), sep="")
     featureIndex <- match(allFeatures, uniqueFeatures)
+    # add an arbitrary "feature" for empty docs
+    if (length(emptyDocs)) {
+        featureIndex <- c(featureIndex, rep(length(allFeatures)+1, length(emptyDocs)))
+        uniqueFeatures <- c(uniqueFeatures, "__TEMPFEATURE__")
+    }
     
     if (verbose) cat("\n")
     
@@ -269,8 +276,8 @@ dfm.tokenizedTexts <- function(x,
                               j = featureIndex, 
                               x = 1L, 
                               dimnames = list(docs = docNames, features = uniqueFeatures))
-    # remove null term
-    dfmresult <- dfmresult[, -match("**_NULL_**", colnames(dfmresult)), drop = FALSE]
+    # remove dummy feature if needed
+    if (length(emptyDocs)) dfmresult <- dfmresult[, -ncol(dfmresult)]
     # construct the dfmSparse type object
     dfmresult <- new("dfmSparse", dfmresult)
     
