@@ -23,8 +23,8 @@ segmentSentence <- function(x, delimiter = NULL, perl = FALSE) {
 # str(segmentParagraph(ukimmigTexts[3]))   # a 12-element char vector
 # 
 # @export
-segmentParagraph <- function(x, delimiter="\\n{2}", perl=FALSE) {
-    tmp <- unlist(strsplit(x, delimiter, perl=perl))
+segmentParagraph <- function(x, delimiter="\\n{2}", perl = FALSE, fixed = FALSE) {
+    tmp <- unlist(strsplit(x, delimiter, fixed = fixed, perl = perl))
     tmp[which(tmp != "")]
 }
 
@@ -56,13 +56,18 @@ segment <- function(x, ...) {
 }
 
 #' @rdname segment
-#' @param what unit of segmentation.  Current options are tokens, sentences, 
-#'   paragraphs, and other.  Segmenting on \code{other} allows segmentation of a
-#'   text on any user-defined value, and must be accompanied by the 
-#'   \code{delimiter} argument.
-#' @param delimiter  delimiter defined as a \code{\link{regex}} for segmentation. Each 
-#'   type has its own default, except \code{other}, which requires a value to be
-#'   specified.
+#' @param what unit of segmentation.  Current options are \code{"tokens"} 
+#'   (default), \code{"sentences"}, \code{"paragraphs"}, \code{"tags"}, and 
+#'   \code{"other"}.  Segmenting on \code{other} allows segmentation of a text 
+#'   on any user-defined value, and must be accompanied by the \code{delimiter} 
+#'   argument.  Segmenting on \code{tags} performs the same function but 
+#'   preserves the tags as a document variable in the segmented corpus.
+#' @param delimiter  delimiter defined as a \code{\link{regex}} for 
+#'   segmentation. Each type has its own default, except \code{other}, which 
+#'   requires a value to be specified.
+#' @param valuetype how to interpret the delimiter: \code{fixed} for exact
+#'   matching; \code{"regex"} for regular expressions; or \code{"glob"} for 
+#'   "glob"-style wildcard patterns
 #' @param perl logical. Should Perl-compatible regular expressions be used?
 #' @export
 #' @examples
@@ -81,35 +86,54 @@ segment.character <- function(x, what=c("tokens", "sentences", "paragraphs", "ta
                                                         ifelse(what=="paragraphs", "\\n{2}", 
                                                                ifelse(what=="tags", "##\\w+\\b", 
                                                                       NULL)))),
+                              valuetype = c("regex", "fixed", "glob"),
                               perl=FALSE,
                               ...) {
     what <- match.arg(what)
+    valuetype <- match.arg(valuetype)
+    if (valuetype == "glob") {
+        # treat as fixed if no glob characters detected
+        if (!sum(stringi::stri_detect_charclass(delimiter, c("[*?]"))))
+            valuetype <- "fixed"
+        else {
+            features <- sapply(delimiter, utils::glob2rx, USE.NAMES = FALSE)
+            valuetype <- "regex"
+        }
+    }
+    
     if (what=="tokens") {
         return(tokenize(x, ...)) 
     } else if (what=="sentences") {
         # warning("consider using tokenize(x, what = \"sentence\") instead.")
         return(lapply(x, segmentSentence, delimiter, perl=perl)) 
     } else if (what=="paragraphs") {
-        return(lapply(x, segmentParagraph, delimiter, perl=perl)) 
+        return(lapply(x, segmentParagraph, delimiter, perl = perl, fixed = (valuetype == "fixed"))) 
     } else if (what=="tags") {
-        return(lapply(x, segmentParagraph, delimiter, perl=perl))         
+        return(lapply(x, segmentParagraph, delimiter, perl = perl, fixed = (valuetype == "fixed")))         
     } else if (what=="other") {
         if (is.null(delimiter))
             stop("For type other, you must supply a delimiter value.")
-        return(lapply(x, segmentParagraph, delimiter, perl=perl))
+        return(lapply(x, segmentParagraph, delimiter, perl = perl, fixed = (valuetype == "fixed"))) 
     }
 }
 
 #' @rdname segment
+#' @param keepdocvars if \code{TRUE}, repeat the docvar values for each
+#'   segmented text; if \code{FALSE}, drop the docvars in the segmented corpus. 
+#'   Dropping the docvars might be useful in order to conserve space or if these 
+#'   are not desired for the segmented corpus.
 #' @export
-#' @note Does not currently record document segments if segmenting a multi-text corpus
-#' into smaller units. For this, use \link{changeunits} instead.
+#' @note Does not currently record document segments if segmenting a multi-text
+#'   corpus into smaller units. For this, use \link{changeunits} instead.
 #' @examples
-#' testCorpus <- corpus("##INTRO This is the introduction. 
-#'                       ##DOC1 This is the first document.  
-#'                       Second sentence in Doc 1.  
-#'                       ##DOC3 Third document starts here.  
-#'                       End of third document.")
+#' testCorpus <- corpus(c("##INTRO This is the introduction. 
+#'                        ##DOC1 This is the first document.  
+#'                        Second sentence in Doc 1.  
+#'                        ##DOC3 Third document starts here.  
+#'                        End of third document.",
+#'                       "##INTRO Document ##NUMBER Two starts before ##NUMBER Three."))
+#' # add a docvar
+#' testCorpus[["serialno"]] <- paste0("textSerial", 1:ndoc(testCorpus))
 #' testCorpusSeg <- segment(testCorpus, "tags")
 #' summary(testCorpusSeg)
 #' texts(testCorpusSeg)
@@ -122,21 +146,49 @@ segment.corpus <- function(x, what = c("tokens", "sentences", "paragraphs", "tag
                                                      ifelse(what=="paragraphs", "\\n{2}", 
                                                             ifelse(what=="tags", "##\\w+\\b", 
                                                                    NULL)))),
+                           valuetype = c("regex", "fixed", "glob"),
                            perl=FALSE,
+                           keepdocvars = TRUE, 
                            ...) {
-    newCorpus <- corpus(unlist(segment(texts(x), what, delimiter, perl=perl, ...)),
+    what <- match.arg(what)
+    valuetype <- match.arg(valuetype)
+    # automatically detect and override valuetype
+    if (gsub("[*?]|\\w|\\s", "", delimiter) != "" & valuetype != "regex") {
+        warning("delimiter looks like it contains a regex", noBreaks. = TRUE)
+    } else if (valuetype == "glob") {
+        # treat as fixed if no glob characters detected
+        if (!sum(stringi::stri_detect_charclass(delimiter, c("[*?]"))))
+            valuetype <- "fixed"
+        else {
+            features <- sapply(delimiter, utils::glob2rx, USE.NAMES = FALSE)
+            valuetype <- "regex"
+        }
+    }
+    
+    segTxt <- segment(texts(x), what, delimiter, perl = perl, valuetype = valuetype, ...)
+    names(segTxt) <- paste0(names(segTxt), ".")
+    
+    newCorpus <- corpus(unlist(segTxt),
                         source = metacorpus(x, "source"),
                         notes = paste0("segment.corpus(", match.call(), ")"))
     
     if (what == "tags") {
-        tagIndex <- gregexpr(delimiter, cattxt <- paste0(texts(x), collapse = ""), perl=perl)[[1]]
+        tagIndex <- gregexpr(delimiter, cattxt <- paste0(texts(x), collapse = ""), 
+                             perl = perl, fixed = (valuetype == "fixed"))[[1]]
         tags <- character()
         length(tags) <- ndoc(newCorpus)
         for (i in 1:length(tagIndex))
             tags[i] <- substr(cattxt, start = tagIndex[i],
                               stop = tagIndex[i] + attr(tagIndex, "match.length")[i] - 1)
+        # remove white space at both ends
+        tags <- stringi::stri_trim_both(tags)
+        # add tag as a docvar
         docvars(newCorpus, "tag") <- tags
     }
+
+    # add repeated versions of remaining docvars
+    if (!is.null(docvars(x)) & keepdocvars)
+        docvars(newCorpus) <- cbind(docvars(newCorpus), lapply(docvars(x), rep, lengths(segTxt)))
     
     newCorpus
 }
@@ -175,20 +227,28 @@ tokenize <- function(x, ...) {
 
 #' @rdname tokenize
 #' @aliases clean
-#' @param what the unit for splitting the text, available alternatives are:
-#'  \describe{ 
-#'  \item{\code{"word"}}{(recommended default) smartest, but slowest, word tokenization method; see \link[stringi]{stringi-search-boundaries} for details.}
-#'  \item{\code{"fasterword"}}{dumber, but faster, word tokenizeation method, \code{\link[stringi]{stri_split_regex}(x, "\\\\s")}}
-#'  \item{\code{"fastestword"}}{dumbest, but fastest, word tokenization method, calls \code{\link[stringi]{stri_split_fixed}(x, " ")}}
-#'  \item{\code{"character"}}{tokenization into individual characters}
-#'  \item{\code{"sentence"}}{sentence segmenter, smart enough to handle some exceptions in English such as "Prof. Plum killed Mrs. Peacock." 
-#'    (but far from perfect).}
-#'  } 
+#' @param what the unit for splitting the text, available alternatives are: 
+#'   \describe{ \item{\code{"word"}}{(recommended default) smartest, but 
+#'   slowest, word tokenization method; see 
+#'   \link[stringi]{stringi-search-boundaries} for details.} 
+#'   \item{\code{"fasterword"}}{dumber, but faster, word tokenizeation method, 
+#'   uses {\link[stringi]{stri_split_charclass}(x, "\\\\p{WHITE_SPACE}")}} 
+#'   \item{\code{"fastestword"}}{dumbest, but fastest, word tokenization method,
+#'   calls \code{\link[stringi]{stri_split_fixed}(x, " ")}} 
+#'   \item{\code{"character"}}{tokenization into individual characters} 
+#'   \item{\code{"sentence"}}{sentence segmenter, smart enough to handle some 
+#'   exceptions in English such as "Prof. Plum killed Mrs. Peacock." (but far 
+#'   from perfect).} }
 #' @param removeNumbers remove tokens that consist only of numbers, but not 
 #'   words that start with digits, e.g. \code{2day}
-#' @param removePunct remove all punctuation
+#' @param removePunct if \code{TRUE}, remove all characters in the Unicode 
+#'   "Punctuation" [P] class
+#' @param removeSymbols if \code{TRUE}, remove all characters in the Unicode 
+#'   "Symbol" [S] class
 #' @param removeTwitter remove Twitter characters \code{@@} and \code{#}; set to
-#'   \code{FALSE} if you wish to eliminate these.
+#'   \code{TRUE} if you wish to eliminate these.
+#' @param removeURL if \code{TRUE}, find and eliminate URLs beginning with
+#'   http(s) -- see section "Dealing with URLs".
 #' @param removeHyphens if \code{TRUE}, split words that are connected by 
 #'   hyphenation and hyphenation-like characters in between words, e.g. 
 #'   \code{"self-storage"} becomes \code{c("self", "storage")}.  Default is 
@@ -205,7 +265,7 @@ tokenize <- function(x, ...) {
 #' @param ngrams integer vector of the \emph{n} for \emph{n}-grams, defaulting 
 #'   to \code{1} (unigrams). For bigrams, for instance, use \code{2}; for 
 #'   bigrams and unigrams, use \code{1:2}.  You can even include irregular 
-#'   sequences such as \code{2:3} for bigrams and trigrams only.  See
+#'   sequences such as \code{2:3} for bigrams and trigrams only.  See 
 #'   \code{\link{ngrams}}.
 #' @param skip integer vector specifying the skips for skip-grams, default is 0 
 #'   for only immediately neighbouring words. Only applies if \code{ngrams} is 
@@ -226,8 +286,17 @@ tokenize <- function(x, ...) {
 #'   intermediate step.  Since \code{tokenize()} is most likely to be used by 
 #'   more technical users, we have set its options to default to minimal 
 #'   intervention. This means that punctuation is tokenized as well, and that 
-#'   nothing is removed by default from the text being tokenized except
+#'   nothing is removed by default from the text being tokenized except 
 #'   inter-word spacing and equivalent characters.
+#' @section Dealing with URLs: URLs are tricky to tokenize, because they contain
+#'   a number of symbols and punctuation characters.  If you wish to remove 
+#'   these, as most people do, and your text contains URLs, then you should set
+#'   \code{what = "fasterword"} and \code{removeURL = TRUE}.  If you wish to
+#'   keep the URLs, but do not want them mangled, then your options are more
+#'   limited, since removing punctuation and symbols will also remove them from
+#'   URLs.  We are working on improving this behaviour.
+#'   
+#'   See the examples below.
 #' @return a \strong{tokenizedText} (S3) object, essentially a list of character
 #'   vectors. If \code{simplify = TRUE} then return a single character vector.
 #' @note This replaces an older function named \code{clean()}, removed from 
@@ -241,62 +310,80 @@ tokenize <- function(x, ...) {
 #' # returned as a list
 #' head(tokenize(inaugTexts[57])[[1]], 10)
 #' # returned as a character vector using simplify=TRUE
-#' head(tokenize(inaugTexts[57], simplify=TRUE), 10)
+#' head(tokenize(inaugTexts[57], simplify = TRUE), 10)
 #' 
 #' # removing punctuation marks and lowecasing texts
-#' head(tokenize(toLower(inaugTexts[57]), simplify=TRUE, removePunct=TRUE), 30)
+#' head(tokenize(toLower(inaugTexts[57]), simplify = TRUE, removePunct = TRUE), 30)
 #' # keeping case and punctuation
-#' head(tokenize(inaugTexts[57], simplify=TRUE), 30)
+#' head(tokenize(inaugTexts[57], simplify = TRUE), 30)
 #' # keeping versus removing hyphens
 #' tokenize("quanteda data objects are auto-loading.", removePunct = TRUE)
 #' tokenize("quanteda data objects are auto-loading.", removePunct = TRUE, removeHyphens = TRUE)
+#' # keeping versus removing symbols
+#' tokenize("<tags> and other + symbols.", removeSymbols = FALSE)
+#' tokenize("<tags> and other + symbols.", removeSymbols = TRUE)
+#' tokenize("<tags> and other + symbols.", removeSymbols = FALSE, what = "fasterword")
+#' tokenize("<tags> and other + symbols.", removeSymbols = TRUE, what = "fasterword")
+#' 
+#' ## examples with URLs - hardly perfect!
+#' txt <- "Repo https://githib.com/kbenoit/quanteda, and www.stackoverflow.com."
+#' tokenize(txt, removeURL = TRUE, removePunct = TRUE)
+#' tokenize(txt, removeURL = FALSE, removePunct = TRUE)
+#' tokenize(txt, removeURL = FALSE, removePunct = TRUE, what = "fasterword")
+#' tokenize(txt, removeURL = FALSE, removePunct = FALSE, what = "fasterword")
+#' 
 #' 
 #' ## MORE COMPARISONS
 #' txt <- "#textanalysis is MY <3 4U @@myhandle gr8 #stuff :-)"
-#' tokenize(txt, removePunct=TRUE)
-#' tokenize(txt, removePunct=TRUE, removeTwitter=TRUE)
-#' #tokenize("great website http://textasdata.com", removeURL=FALSE)
-#' #tokenize("great website http://textasdata.com", removeURL=TRUE)
+#' tokenize(txt, removePunct = TRUE)
+#' tokenize(txt, removePunct = TRUE, removeTwitter = TRUE)
+#' #tokenize("great website http://textasdata.com", removeURL = FALSE)
+#' #tokenize("great website http://textasdata.com", removeURL = TRUE)
 #' 
 #' txt <- c(text1="This is $10 in 999 different ways,\n up and down; left and right!", 
 #'          text2="@@kenbenoit working: on #quanteda 2day\t4ever, http://textasdata.com?page=123.")
-#' tokenize(txt, verbose=TRUE)
-#' tokenize(txt, removeNumbers=TRUE, removePunct=TRUE)
-#' tokenize(txt, removeNumbers=FALSE, removePunct=TRUE)
-#' tokenize(txt, removeNumbers=TRUE, removePunct=FALSE)
-#' tokenize(txt, removeNumbers=FALSE, removePunct=FALSE)
-#' tokenize(txt, removeNumbers=FALSE, removePunct=FALSE, removeSeparators=FALSE)
+#' tokenize(txt, verbose = TRUE)
+#' tokenize(txt, removeNumbers = TRUE, removePunct = TRUE)
+#' tokenize(txt, removeNumbers = FALSE, removePunct = TRUE)
+#' tokenize(txt, removeNumbers = TRUE, removePunct = FALSE)
+#' tokenize(txt, removeNumbers = FALSE, removePunct = FALSE)
+#' tokenize(txt, removeNumbers = FALSE, removePunct = FALSE, removeSeparators = FALSE)
+#' tokenize(txt, removeNumbers = TRUE, removePunct = TRUE, removeURL = TRUE)
 #' 
 #' # character level
-#' tokenize("Great website: http://textasdata.com?page=123.", what="character")
-#' tokenize("Great website: http://textasdata.com?page=123.", what="character", 
-#'          removeSeparators=FALSE)
+#' tokenize("Great website: http://textasdata.com?page=123.", what = "character")
+#' tokenize("Great website: http://textasdata.com?page=123.", what = "character", 
+#'          removeSeparators = FALSE)
 #' 
 #' # sentence level         
 #' tokenize(c("Kurt Vongeut said; only assholes use semi-colons.", 
 #'            "Today is Thursday in Canberra:  It is yesterday in London.", 
 #'            "Today is Thursday in Canberra:  \nIt is yesterday in London.",
-#'            "To be?  Or\not to be?"), 
+#'            "To be?  Or\nnot to be?"), 
 #'           what = "sentence")
 #' tokenize(inaugTexts[c(2,40)], what = "sentence", simplify = TRUE)
 #' 
-#' # creating ngrams
+#' # removing features (stopwords) from tokenized texts
 #' txt <- toLower(c(mytext1 = "This is a short test sentence.",
-#'                 mytext2 = "Short.",
-#'                 mytext3 = "Short, shorter, and shortest."))
+#'                  mytext2 = "Short.",
+#'                  mytext3 = "Short, shorter, and shortest."))
 #' tokenize(txt, removePunct = TRUE)
 #' removeFeatures(tokenize(txt, removePunct = TRUE), stopwords("english"))
+#' 
+#' # ngram tokenization
 #' tokenize(txt, removePunct = TRUE, ngrams = 2)
-#' tokenize(txt, removePunct = TRUE, ngrams = 1:2)
 #' tokenize(txt, removePunct = TRUE, ngrams = 2, skip = 1, concatenator = " ")
+#' tokenize(txt, removePunct = TRUE, ngrams = 1:2)
+#' # removing features from ngram tokens
 #' removeFeatures(tokenize(txt, removePunct = TRUE, ngrams = 1:2), stopwords("english"))
 tokenize.character <- function(x, what=c("word", "sentence", "character", "fastestword", "fasterword"),
-                               removeNumbers = FALSE, 
+                               removeNumbers = FALSE,
                                removePunct = FALSE,
+                               removeSymbols = FALSE,
                                removeSeparators = TRUE,
                                removeTwitter = FALSE,
                                removeHyphens = FALSE,
-                               # removeURL = TRUE,
+                               removeURL = FALSE,
                                ngrams = 1L,
                                skip = 0L,
                                concatenator = "_",
@@ -324,58 +411,81 @@ tokenize.character <- function(x, what=c("word", "sentence", "character", "faste
     if (verbose) cat("  ...tokenizing texts")
     startTimeTok <- proc.time()
     
-    if (what == "fasterword" | what == "fastestword") {
-        
-        if (verbose & removeNumbers==TRUE) cat(", removing numbers")
-        if (verbose & removePunct==TRUE) cat(", removing punctuation")
-        regexToEliminate <- paste0(ifelse(removeNumbers, "\\b\\d+\\b|", ""),
-                                   ifelse(removePunct, paste0("(?![", ifelse(removeTwitter, "_", "@#_"), "])[[:punct:]]"), "|"))
-        if (regexToEliminate != "|")
-            result <- stri_replace_all_regex(result, regexToEliminate, "")
-        
-        if (verbose & removePunct==TRUE) cat(", ", what, "tokenizing", sep="")
-        if (what=="fastestword")
-            result <- stringi::stri_split_fixed(result, " ")
-        else if (what=="fasterword")
-            result <- stringi::stri_split_regex(result, "\\s")
-        result <- lapply(result, function(x) x <- x[which(x != "")])
-        
-    } else if (what == "character") {
-        
-        # note: does not implement removeNumbers
-        result <- stringi::stri_split_boundaries(result, type = "character")
-        if (removePunct) {
-            if (verbose) cat("   ...removing punctuation.\n")
-            result <- lapply(result, stringi::stri_replace_all_charclass, "[\\p{P}\\p{S}]", "")
-            result <- lapply(result, function(x) x <- x[which(x != "")])
-        } 
-        if (removeSeparators) {
-            if (verbose) cat("   ...removing separators.\n")
-            result <- lapply(result, function(x) x[!stringi::stri_detect_regex(x, "^\\s$")])
-        }
-        
-        
-    } else if (what == "word") {
+    if (grepl("word$", what)) {
         
         # to preserve intra-word hyphens, replace with _hy_
         if (!removeHyphens & removePunct)
             result <- stri_replace_all_regex(result, "(\\b)[\\p{Pd}](\\b)", "$1_hy_$2")
         else if (removeHyphens)
             result <- stri_replace_all_regex(result, "(\\b)[\\p{Pd}](\\b)", "$1 $2")
+        
+        if (removeURL) {
+            if (verbose & removeURL) cat(", removing URLs")
+            URLREGEX <- "https?:\\/\\/(www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{2,256}\\.[a-z]{2,4}\\b([-a-zA-Z0-9@:%_\\+.~#?&//=]*)"
+            result <- stri_replace_all_regex(result, URLREGEX, "")
+        }
+
+        if (what == "fasterword" | what == "fastestword") {
+        
+            if (verbose & removeNumbers==TRUE) cat(", removing numbers")
+            if (verbose & removePunct==TRUE) cat(", removing punctuation")
+            if (verbose & removeSymbols==TRUE) cat(", removing symbols")
+            regexToEliminate <- paste(ifelse(removeNumbers, "\\b\\d+\\b", ""),
+                                      ifelse(removePunct, paste0("(?![", ifelse(removeTwitter, "_", "@#_"),  "])[\\p{P}]"), ""),
+                                      ifelse(removeSymbols, "[\\p{S}]", ""),
+                                      sep = "|")
+            # cat("\n..", regexToEliminate, "..\n", sep = "")
+            regexToEliminate <- gsub("^\\|+", "", regexToEliminate)
+            regexToEliminate <- gsub("\\|+$", "", regexToEliminate)
+            # cat("\n..", regexToEliminate, "..\n", sep = "")
+            if (gsub("|", "", regexToEliminate, fixed = TRUE) != "")
+                result <- stri_replace_all_regex(result, regexToEliminate, "")
             
-        result <- stringi::stri_split_boundaries(result, 
-                                                 type = "word", 
-                                                 skip_word_none = removePunct, # this is what obliterates currency symbols, Twitter tags, and URLs
-                                                 skip_word_number = removeNumbers) # but does not remove 4u, 2day, etc.
+            if (verbose & removePunct==TRUE) cat(", ", what, " tokenizing", sep="")
+            if (what=="fastestword")
+                result <- stringi::stri_split_fixed(result, " ")
+            else if (what=="fasterword")
+                result <- stringi::stri_split_charclass(result, "\\p{WHITE_SPACE}")
+            result <- lapply(result, function(x) x <- x[which(x != "")])
+            
+            # if (removeURL)
+            #     result <- lapply(result, function(x) x <- x[-which(substring(x, 1, 4) == "http")])
+
+        } else {
+            result <- stringi::stri_split_boundaries(result, 
+                                                     type = "word", 
+                                                     skip_word_none = (removePunct | removeSymbols), # this is what obliterates currency symbols, Twitter tags, and URLs
+                                                     skip_word_number = removeNumbers) # but does not remove 4u, 2day, etc.
+            # remove separators if option is TRUE
+            if (removeSeparators & !removePunct) {
+                if (verbose) cat("\n  ...removing separators.")
+                result <- lapply(result, function(x) x[!stri_detect_regex(x, "^\\s$")])
+            }
+        }
+        
         # put hyphens back the fast way
         if (!removeHyphens & removePunct)
             result <- lapply(result, stri_replace_all_fixed, "_hy_", "-")
-        # remove separators if option is TRUE
-        if (removeSeparators & !removePunct) {
-            if (verbose) cat("\n   ...removing separators.")
-            result <- lapply(result, function(x) x[!stri_detect_regex(x, "^\\s$")])
-        }
 
+    } else if (what == "character") {
+        
+        # note: does not implement removeNumbers
+        result <- stringi::stri_split_boundaries(result, type = "character")
+        if (removePunct) {
+            if (verbose) cat("   ...removing punctuation.\n")
+            result <- lapply(result, stringi::stri_replace_all_charclass, "[\\p{P}]", "")
+            result <- lapply(result, function(x) x <- x[which(x != "")])
+        } 
+        if (removeSymbols) {
+            if (verbose) cat("   ...removing symbols.\n")
+            result <- lapply(result, stringi::stri_replace_all_charclass, "[\\p{S}]", "")
+            result <- lapply(result, function(x) x <- x[which(x != "")])
+        } 
+        if (removeSeparators) {
+            if (verbose) cat("   ...removing separators.\n")
+            result <- lapply(result, function(x) x[!stringi::stri_detect_regex(x, "^\\p{Z}$")])
+        }
+        
     } else if (what == "sentence") {
         if (verbose) cat("\n   ...separating into sentences.")
         
@@ -384,13 +494,17 @@ tokenize.character <- function(x, what=c("word", "sentence", "character", "faste
         findregex <- paste0("\\b(", exceptions, ")\\.")
         result <- stri_replace_all_regex(result, findregex, "$1_pd_", vectorize_all = FALSE)
 
+        ## remove newline chars 
+        result <- lapply(result, stringi::stri_replace_all_fixed, "\n", " ")
+
+        ## perform the tokenization
         result <- stringi::stri_split_boundaries(result, type = "sentence")
-        ## remove newline chars and trailing spaces for sentence tokenization
-        result <- lapply(result, stringi::stri_replace_all_fixed, "\n", "")
-        result <- lapply(result, stringi::stri_trim_right)
         # remove any "sentences" that were completely blanked out
         result <- lapply(result, function(x) x <- x[which(x != "")])
-        
+
+        # trim trailing spaces
+        result <- lapply(result, stringi::stri_trim_right)
+
         # replace the non-full-stop "." characters
         result <- lapply(result, stri_replace_all_fixed, "_pd_", ".")
 
